@@ -32,20 +32,17 @@ module Crypto.Nonce
 
 import Control.Monad (liftM)
 import Control.Monad.IO.Class (MonadIO, liftIO)
-import Crypto.Random
-import Data.Tuple (swap)
+import qualified System.Entropy as Entropy
 import Data.Typeable (Typeable)
 
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Base64.URL as B64URL
-import qualified Data.IORef as I
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as TE
 
 
 -- | An encapsulated nonce generator.
-data Generator =
-  G (I.IORef SystemDRG)
+data Generator = G Entropy.CryptHandle
   deriving (Typeable)
 
 instance Show Generator where
@@ -54,12 +51,12 @@ instance Show Generator where
 
 -- | Create a new nonce generator using the system entropy.
 new :: MonadIO m => m Generator
-new = liftM G . liftIO $ getSystemDRG >>= I.newIORef
+new = liftM G . liftIO $ Entropy.openHandle
 
 
 -- | (Internal) Generate the given number of bytes from the DRG.
 genBytes :: MonadIO m => Int -> Generator -> m B.ByteString
-genBytes n (G v) = liftIO $ I.atomicModifyIORef v $ swap . randomBytesGenerate n
+genBytes n (G v) = liftIO $ Entropy.hGetEntropy v n
 
 
 -- | Generate a 128 bit nonce as a 'B.ByteString' of 16 bytes.
